# Changelog - Setup-Konsolidierung v4.0.0

## Übersicht der Änderungen

Diese Version konsolidiert alle bisherigen Installationsskripte in ein einziges, benutzerfreundliches Setup-System.

## 🔄 Strukturelle Änderungen

### Neue Dateien

- **`setup.sh`** - Konsolidiertes Hauptinstallationsskript mit Menüführung
- **`systemd/`** - Neues Verzeichnis für alle systemd-Service-Dateien
  - `systemd/myp-https.service`
  - `systemd/myp-kiosk.service`
  - `systemd/kiosk-watchdog.service`
  - `systemd/kiosk-watchdog-python.service`
- **`docs/SETUP_ANLEITUNG.md`** - Detaillierte Anleitung für das neue Setup-System

### Entfernte Dateien

- **`combined.sh`** - Konsolidiert in `setup.sh`
- **`installer.sh`** - Konsolidiert in `setup.sh`
- Service-Dateien aus dem Stammverzeichnis (verschoben nach `systemd/`)

### Aktualisierte Dateien

- **`README.md`** - Vollständig überarbeitet für neues Setup-System
- **Version erhöht auf 4.0.0**

## 🚀 Neue Features

### Menügeführte Installation

Das neue `setup.sh` bietet vier verschiedene Installationsmodi:

1. **Nur Abhängigkeiten installieren**

   - Python, Node.js, SSL-Zertifikate
   - Anwendungsdeployment
   - Minimaler Funktionstest
   - Ideal für Entwicklung
2. **Finale Kiosk-Installation**

   - Vollständige Produktionsinstallation
   - Desktop-Environment-Entfernung
   - Kiosk-Benutzer-Setup
   - Service-Aktivierung
3. **Nur Services installieren**

   - Service-Updates ohne Systemänderungen
   - Systemd-Konfiguration neu laden
   - Optional: Services aktivieren
4. **System-Test**

   - HTTPS-Verbindungstest
   - SSL-Zertifikat-Validierung
   - Service-Status-Überprüfung
   - Systemressourcen-Monitoring

### Verbesserte Modularität

- **Getrennte Installationsphasen** für bessere Kontrolle
- **Fallback-Mechanismen** für robuste Installation
- **Intelligente Abhängigkeitserkennung**
- **Automatische Fehlerbehandlung**

### Erweiterte Logging-Funktionen

- **Farbkodierte Ausgabe** für bessere Lesbarkeit
- **Detailliertes Installationslog** (`/var/log/myp-install.log`)
- **Strukturierte Fehlermeldungen**
- **Fortschrittsanzeigen**

## 🔧 Technische Verbesserungen

### SSL-Zertifikat-Management

- **Automatische Mercedes-Zertifikat-Installation**
- **DER-zu-PEM Konvertierung**
- **Zertifikat-Validierung**
- **Ablaufüberwachung**

### Service-Management

- **Zentralisierte Service-Dateien** in `systemd/` Verzeichnis
- **Automatische systemd-Konfiguration**
- **Service-Abhängigkeiten optimiert**
- **Watchdog-Integration verbessert**

### Systemoptimierung

- **Minimale X11-Installation** für Kiosk-Modus
- **Browser-Fallback-Mechanismen** (Chromium → Firefox)
- **Speicher-Überwachung** und automatische Bereinigung
- **Netzwerk-Konnektivitätsprüfung**

## 📊 Kompatibilität

### Unterstützte Systeme

- **Debian 11+** (Bullseye und neuer)
- **Raspberry Pi OS** (alle aktuellen Versionen)
- **Ubuntu 20.04+** (experimentell)

### Rückwärtskompatibilität

- **Bestehende Installationen** können mit Option 3 aktualisiert werden
- **Datenbank-Migration** automatisch
- **Konfigurationsdateien** bleiben erhalten

## 🛡️ Sicherheitsverbesserungen

### Systemd-Härtung

- **NoNewPrivileges** für alle Services
- **ProtectSystem=strict** für Dateisystem-Schutz
- **Minimale Capabilities** für privilegierte Ports
- **Service-Isolation** verbessert

### SSL/TLS-Optimierung

- **Starke Cipher-Suites** konfiguriert
- **TLS 1.2+ erzwungen**
- **Zertifikat-Rotation** automatisiert
- **Corporate-Zertifikat-Integration**

## 🔄 Migration von alten Installationen

### Automatische Migration

```bash
# Backup erstellen
sudo cp -r /opt/myp /opt/myp.backup

# Neue Installation
sudo ./setup.sh
# Option 3: Nur Services installieren
```

### Manuelle Schritte (falls erforderlich)

1. **Service-Dateien aktualisieren**
2. **Systemd-Konfiguration neu laden**
3. **Services neu starten**
4. **Funktionstest durchführen**

## 📈 Performance-Verbesserungen

### Installation

- **Parallele Paket-Downloads** wo möglich
- **Intelligente Abhängigkeitsauflösung**
- **Reduzierte Installationszeit** durch Optimierungen
- **Bessere Fehlerbehandlung** verhindert Neuinstallationen

### Laufzeit

- **Watchdog-Optimierung** für geringeren Ressourcenverbrauch
- **Browser-Cache-Management** automatisiert
- **Speicher-Überwachung** mit automatischer Bereinigung
- **Service-Restart-Logik** verbessert

## 🐛 Behobene Probleme

### Installation

- **npm-Installation** robuster mit Fallback-Strategien
- **SSL-Zertifikat-Generierung** zuverlässiger
- **Service-Abhängigkeiten** korrekt konfiguriert
- **Berechtigungsprobleme** behoben

### Kiosk-Modus

- **Browser-Autostart** zuverlässiger
- **Display-Erkennung** verbessert
- **Crash-Recovery** automatisiert
- **Speicher-Leaks** behoben

## 📚 Dokumentation

### Neue Dokumentation

- **`docs/SETUP_ANLEITUNG.md`** - Vollständige Setup-Anleitung
- **Erweiterte README.md** mit neuen Installationsoptionen
- **Inline-Kommentare** in `setup.sh` für bessere Wartbarkeit

### Aktualisierte Dokumentation

- **API-Dokumentation** überarbeitet
- **Fehlerbehebungsguide** erweitert
- **Konfigurationsoptionen** dokumentiert

## 🔮 Zukunftsausblick

### Geplante Features (v4.1.0)

- **Web-basiertes Setup-Interface**
- **Remote-Installation** über SSH
- **Automatische Updates** über Git
- **Monitoring-Dashboard** für Systemstatus

### Langfristige Ziele

- **Container-basierte Deployment** (Docker)
- **Kubernetes-Integration** für Skalierung
- **Cloud-Deployment-Optionen**
- **Multi-Tenant-Unterstützung**

## 📞 Support und Feedback

### Neue Support-Kanäle

- **System-Test-Funktion** (Option 4) für Selbstdiagnose
- **Automatische Log-Sammlung** für Support-Anfragen
- **Strukturierte Fehlermeldungen** für bessere Problemlösung

### Feedback erwünscht

- **Installation-Erfahrungen** auf verschiedenen Systemen
- **Performance-Messungen** in Produktionsumgebungen
- **Feature-Requests** für zukünftige Versionen

---

**Datum**: 01.06.2025
**Version**: 4.0.0
**Typ**: Major Release - Setup-Konsolidierung
**Kompatibilität**: Rückwärtskompatibel mit automatischer Migration
